//	IcoCreator.cpp - Class which takes up a number of input files and creates an
//				   ico file from them.
//	----------------------------------------------------------------------------
//	This file is part of png2ico, a simple .ico file creator.
//	Copyright (c) 2011 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "IcoCreator.h"

//------------------------------------------------------------------------------
IcoCreator::IcoCreator()
{

}

//------------------------------------------------------------------------------
IcoCreator::~IcoCreator()
{

}

//------------------------------------------------------------------------------
void IcoCreator::addFile(const File& phil)
{
	inputImages.add(phil);
}

//------------------------------------------------------------------------------
void IcoCreator::clearFiles()
{
	inputImages.clear();
}

//------------------------------------------------------------------------------
bool IcoCreator::createIco(const File& path)
{
	int i;
	bool retval = true;

	if((path != File::nonexistent) && (inputImages.size() > 0))
	{
		uint8 buffer[16384];
		uint16 *bufPos16;
		FileOutputStream outFile(path);
		uint32 imageOffset = 6 + (inputImages.size() * 16);

		//Write ICONDIR structure.

		//Reserved. Must be 0.
		bufPos16 = (uint16 *)buffer;
		*bufPos16 = 0;

		//Image type. 1 == ico.
		bufPos16 = (uint16 *)(buffer+2);
		*bufPos16 = 1;

		//Number of images in file.
		bufPos16 = (uint16 *)(buffer+4);
		*bufPos16 = (uint16)(inputImages.size()); 

		outFile.write(buffer, 6);

		//Write the ICONDIRENTRY structures, 1 for each image.
		for(i=0;i<inputImages.size();++i)
		{
			FileInputStream inputFile(inputImages[i]);
			PNGImageFormat pngReader;
			Image im;
			uint32 imageSize;

			if(pngReader.canUnderstand(inputFile))
			{
				inputFile.setPosition(0);
				im = pngReader.decodeImage(inputFile);

				if((im.getWidth() > 256) || (im.getHeight() > 256))
				{
					retval = false;
					break;
				}
				else
				{
					//Image width.
					if(im.getWidth() == 256)
						buffer[0] = 0;
					else
						buffer[0] = (uint8)im.getWidth();

					//Image height.
					if(im.getHeight() == 256)
						buffer[1] = 0;
					else
						buffer[1] = (uint8)im.getHeight();

					//Colour palette.
					buffer[2] = 0;

					//Reserved. Must be 0.
					buffer[3] = 0;

					//Colour planes.
					bufPos16 = (uint16 *)(buffer+4);
					*bufPos16 = 1;

					//Bits per pixel.
					bufPos16 = (uint16 *)(buffer+6);
					*bufPos16 = 8;

					//Size of image data in bytes.
					{
						uint32 *dataPos = (uint32 *)(buffer + 8);
						imageSize = (uint32)inputFile.getTotalLength();

						*dataPos = imageSize;
					}

					//Image offset.
					{
						uint32 *dataPos = (uint32 *)(buffer + 12);

						*dataPos = imageOffset;
						imageOffset += imageSize;
					}

					outFile.write(buffer, 16);
				}
			}
			else
			{
				retval = false;
				break;
			}
		}

		if(retval)
		{
			//Write the images. We just copy the png data in its entirety.
			for(i=0;i<inputImages.size();++i)
			{
				int bytesRead;
				uint8 buffer[16384];
				FileInputStream inputFile(inputImages[i]);

				while(!inputFile.isExhausted())
				{
					bytesRead = inputFile.read(buffer, 16384);
					outFile.write(buffer, bytesRead);
				}
			}
		}
	}

	return retval;
}
